--[[
Name: MythicKeystoneHelper

Copyright 2018 Amegez

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
]]

local addon = {}
local addonName = "MythicKeystoneHelper"

if not LibStub then
	print(addonName.." requires LibStub.")
	return
end

local ldb = LibStub:GetLibrary("LibDataBroker-1.1", true)

local plugin = ldb:NewDataObject(addonName, {
	type = "data source",
	text = "0",
	icon = "Interface\\ICONS\\INV_Misc_Key_15",
})

function plugin.OnClick(self, button)
	if button == "LeftButton" then
		MKH_createMythicGroup()
	else
		if(ChallengesKeystoneFrame and ChallengesKeystoneFrame:IsShown())then
			MKH_slotKey()
		else
			MKH_searchMythicGroup()
		end
	end
end

do
	function plugin.OnTooltipShow(tt)
		local idKey,lvl,a1,a2,a3,a4 = MKH_getBagKeystoneInfo()
		tt:AddLine("|cff1eff00"..addonName.."|r")
		
		if idKey then
			local activityID = ACTIVITY_IDS[idKey]
			local name, _, timeLimit = C_ChallengeMode.GetMapUIInfo(idKey);
			local dmgPct, healthPct = C_ChallengeMode.GetPowerLevelDamageHealthMod(tonumber(lvl));
			tt:AddLine("|cffa335ee"..name.." +"..lvl.."|r")
			tt:AddLine("|cffff8000"..string.format(L["Timer"],(timeLimit/60), dmgPct, healthPct).."|r")
			if (a1 ~= "0") then
				local name, description = C_ChallengeMode.GetAffixInfo(tonumber(a1))
				tt:AddLine("|cff0070dd"..name.."|r|cff9d9d9d: "..description.."|r")
			end
			if (a2 ~= "0") then
				local name, description = C_ChallengeMode.GetAffixInfo(tonumber(a2))
				tt:AddLine("|cff0070dd"..name.."|r|cff9d9d9d: "..description.."|r")
			end
			if (a3 ~= "0") then	
				local name, description = C_ChallengeMode.GetAffixInfo(tonumber(a3))
				tt:AddLine("|cff0070dd"..name.."|r|cff9d9d9d: "..description.."|r")
			end
			if (a4 ~= "0") then
				local name, description = C_ChallengeMode.GetAffixInfo(tonumber(a4))
				tt:AddLine("|cff0070dd"..name.."|r|cff9d9d9d: "..description.."|r")
			end
			
			local player = UnitName("player")
			if MythicKeystoneHelperDB.profiles[player] then
				
			end
		else
			tt:AddLine("|cffa335ee"..L["NoKey"].."|r")
		end
		
		local otherKeys = ""
		local player = string.format("%s-%s",UnitFullName("player"))
		for plyer, key in pairs (MythicKeystoneHelperDB.profiles) do
			if plyer ~= player or not key then
				if otherKeys ~= "" then
					otherKeys = otherKeys..", "
				end
				otherKeys = otherKeys..string.format("%s : |cffa335ee%s|r",plyer,key)
			end
		end	
		tt:AddLine(otherKeys)
	end
end

local f = CreateFrame("Frame")
f:SetScript("OnEvent", function(e)
	if (MythicKeystoneHelperDB == nil) then 
		MythicKeystoneHelperDB = {}
	end
	if (MythicKeystoneHelperDB.profiles == nil) then
		MythicKeystoneHelperDB.profiles = {}
	end
	if (MythicKeystoneHelperDB.config == nil) then
		MythicKeystoneHelperDB.config = {}
		MythicKeystoneHelperDB.config.filter = true
	end
		
	local icon = LibStub("LibDBIcon-1.0", true)
	if not icon then return end
	icon:Register(addonName, plugin, MythicKeystoneHelperDB)
	
	local idKey,lvl = MKH_getBagKeystoneInfo()
	
	if idKey then
		local activityID = ACTIVITY_IDS[idKey]
		local name = C_ChallengeMode.GetMapUIInfo(idKey);
		local player = string.format("%s-%s",UnitFullName("player"))
		MythicKeystoneHelperDB.profiles[player] = name.." +"..lvl
	else
		if MythicKeystoneHelperDB.profiles[player] then
			MythicKeystoneHelperDB.profiles[player] = nil
		end
	end
	
	MKH_initFilterButton()
	print(addonName.." loaded.")
end)
f:RegisterEvent("PLAYER_ENTERING_WORLD")
f:RegisterEvent("PLAYER_LOGOUT")

function MKH_getBagKeystoneInfo()
	for bag=0, NUM_BAG_SLOTS do
	   for bagSlots=1, GetContainerNumSlots(bag) do
		  local itemlink = GetContainerItemLink(bag, bagSlots)
		  if (itemlink) then
			 local printable=gsub(itemlink, "\124", "\124\124")
			 local _,_,_,_,itemInfo = strsplit("|",printable)
			 local itemType,itemID,idKey,lvl,affix1,affix2,affix3,affix4 = strsplit(":",itemInfo)
			 if(itemType == "Hkeystone") then
				return tonumber(idKey),lvl,affix1,affix2,affix3,affix4,bag,bagSlots
			 end
		  end
	   end
	end
	return nil,nil,nil,nil,nil,nil,nil,nil
end

function MKH_initFilterButton()
	MKH_FilterButton = CreateFrame("CheckButton", "MKH_FilterButton_GlobalName", LFGListFrame.SearchPanel, "ChatConfigCheckButtonTemplate")
	local hAlign = -5
	if ElvUI then
	   hAlign = 0
	end
	MKH_FilterButton:SetPoint("TOPLEFT", LFGListFrame.SearchPanel.RefreshButton, "TOPLEFT", -25, hAlign)
	MKH_FilterButton:SetChecked(MythicKeystoneHelperDB.config.filter)
	MKH_FilterButton.tooltip = "|cff1eff00"..addonName.."|r"..L["Filter"]
	MKH_FilterButton:SetScript("OnClick", 
		function()
			MythicKeystoneHelperDB.config.filter = not MythicKeystoneHelperDB.config.filter
		end
	);
end

function MKH_tContains(table, item)
   local index = 1;
   while table[index] do
      if ( item == table[index] ) then
         return 1;
      end
      index = index + 1;
   end
   return nil;
end

function MKH_createMythicGroup()
	local idKey = MKH_getBagKeystoneInfo()
	if idKey then
		local activityID = ACTIVITY_IDS[idKey]
		MKH_openGroupCreateFrame(activityID)
		SetDungeonDifficultyID(23)
	else
		MKH_displayMessage(L["NoKey"])
		MKH_searchMythicGroup()
	end
end

function MKH_searchMythicGroup()
	MKH_openGroupSearchFrame()
	SetDungeonDifficultyID(23)
end

function MKH_slotKey()
	local _,_,_,_,_,_,b,s = MKH_getBagKeystoneInfo()
	if b then
		PickupContainerItem(b,s)
		UseContainerItem(b,s)
	else
		MKH_displayMessage(L["NoKey"])
		if not GroupFinderFrame:IsVisible() then
			PVEFrame_ShowFrame("GroupFinderFrame")
		end
	end
end

function MKH_openGroupSearchFrame()
	if not GroupFinderFrame:IsVisible() then
	   PVEFrame_ShowFrame("GroupFinderFrame")
	end
	C_Timer.After(0.25,function()
		GroupFinderFrameGroupButton4:Click()
		C_Timer.After(0.25,function()
			LFGListCategorySelection_SelectCategory(LFGListFrame.CategorySelection,2,0)
			LFGListFrame.CategorySelection.FindGroupButton:Click()
			C_Timer.After(0.25,function()
				LFGListFrame.SearchPanel.SearchingSpinner:Hide()
				LFGListFrame.SearchPanel.ScrollFrame.NoResultsFound:Show()
			end)
		end)
	end)
end


function MKH_openGroupCreateFrame(activityID)
	if not GroupFinderFrame:IsVisible() then
	   PVEFrame_ShowFrame("GroupFinderFrame")
	end
	C_Timer.After(0.25,function()
		GroupFinderFrameGroupButton4:Click()
		C_Timer.After(0.25,function()
			LFGListCategorySelection_SelectCategory(LFGListFrame.CategorySelection,2,0)
			LFGListFrame.CategorySelection.StartGroupButton:Click()
			C_Timer.After(0.25,function()
				  LFGListEntryCreation_Select(LFGListFrame.EntryCreation, 0, 0, 0, activityID)
				  LFGListFrame.EntryCreation.Name:Click()
			end)
		end)
	end)
end

hooksecurefunc("LFGListUtil_SetSearchEntryTooltip", function(tooltip, resultID, autoAcceptOption)
	local id, _, _, _, _, _, _, _, _, _, _, _, _ ,numMembers = C_LFGList.GetSearchResultInfo(resultID)
	local roles = {}
	roles["TANK"] = ""
	roles["HEALER"] = ""
	roles["DAMAGER"] = ""
	for i=1, numMembers do
		local role, class, classLocalized = C_LFGList.GetSearchResultMemberInfo(id, i)
		
		if roles[role] == "" then
			roles[role] = roles[role].."|c"..CLASS_COLOR[class]..classLocalized.."|r"
		else
			roles[role] = roles[role]..", |c"..CLASS_COLOR[class]..classLocalized.."|r"
		end
	end
	tooltip:AddLine(" ")
	tooltip:AddLine("|cff9d9d9dTANK: |r"..roles["TANK"])
	tooltip:AddLine("|cff9d9d9dHEAL: |r"..roles["HEALER"])
	tooltip:AddLine("|cff9d9d9dDPS: |r"..roles["DAMAGER"])
	tooltip:Show()
end)

hooksecurefunc("LFGListUtil_SortSearchResults", function(results)
	if (LFGListFrame.SearchPanel.results and MythicKeystoneHelperDB.config.filter) then
		local title = LFGListFrame.SearchPanel.SearchBox:GetText()
		local j = 1
		local newResults = {}
		for k,v in pairs(results) do
			local id , _, name, _, _, _, _, _, _, _, _, _, _ ,numMembers = C_LFGList.GetSearchResultInfo(v)
			
			local addToList = true
			
			if (title ~= "") then
				if (string.match(name,title.." ") or string.match(name,title.."$")) then
					addToList = true
				else
					addToList = false
				end
			end
			
			if (addToList) then
				addToList = MKH_isRemainingRolesForGroup(id,numMembers)
			end
			
			if (addToList) then
				table.insert(newResults,results[j])
			end
			
			j = j + 1
		end
		LFGListFrame.SearchPanel.results = newResults
	end
end)

function MKH_isRemainingRolesForGroup(id,numMembers)
   local pGR = MKH_getPlayerGroupRoles()
   local sGR = MKH_getGroupRoles(id, numMembers)
   for i = 1, #pGR do
      if MKH_tContains(sGR,pGR[i]) then
         return false
      end
   end
   return true
end

function MKH_getPlayerGroupRoles()
   local groupRole = {}
   local index = 1
   local currentRole = ""
   local d=3
   while index < 5 do
      local unitID    = "party"..index
      if UnitExists(unitID) then 
         currentRole = UnitGroupRolesAssigned(unitID)
         if currentRole == "DAMAGER" then
            currentRole = currentRole..d
            d = d - 1
         end
         table.insert(groupRole,currentRole)
      end
      index = index + 1
   end
   currentRole = GetSpecializationRole(GetSpecialization())
	if currentRole == "DAMAGER" then
		currentRole = currentRole..d
		d= d - 1
   end
   table.insert(groupRole,currentRole)
   return groupRole
end

function MKH_getGroupRoles(id, numMembers)
   local groupRole = {}
   local d = 1
   for i = 1, numMembers do
      local currentRole = C_LFGList.GetSearchResultMemberInfo(id, i)
      if currentRole == "DAMAGER" then
         currentRole = currentRole..d
         d = d + 1
      end
      table.insert(groupRole,currentRole)
   end
   return groupRole
end

function MKH_displayMessage(msg)
	print("|cffff8000"..addonName.." : "..msg.."|r.")
end